// Global Header file of the CForth LowPower project *************************
//
// Author and (c)2020-24 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.c", "main.h", "cforthLP.c"
// and "cforthLP.h"
// Other depending material is published, copyrighted and licensed
// by STMicroelectronics and Segger GmbH
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#ifndef GLOBALDEFINES
#define GLOBALDEFINES

#define REVNUM  11 			//describes firmware revision
#define HARDWARE_VERSION 1	//describes project specific hardware version

#//define MODUL_24//24pin module with quarz osc and RS-232 onboard.Not published
				  //LED pin is the only difference

//#define CLOCK_14M //SYSCLK=14.7456MHz,CPU=921.6kHz/14.7456MHz switchable
//				ifndef:SYSCLK=3.6864MHz, CPU=921.6kHz/3.6864MHz switchable

//attributes of KernelOperators
//attribute entries are merged with some other functional flags !!
#define A_IME			0x10000 //ATTRIBUTE_IMMEDIATE, cannot be compiled
#define A_CPO			0x20000 //ATTRIBUTE_COMPILEONLY,refuse work interpreting
#define A_BAK			0x40000 //may be parsed in background at executing state
#define A_CI			(A_CPO | A_IME) //COMPILEONLY IMMEDIATE
								//performs complex operations during compilation
#define A_IB			(A_IME | A_BAK)
#define A_RUN			0x80000 //ATTRIBUTE_RUNTIME,
					//compiled by other operators for special runtime behaviour
					//no direct user access, essential for viewing SEE and STEP
//only attribute of UserToken
#define A_VARCON		0x0C000000

#define ERR_STRINGTOKEN	1
#define ERR_SYNTAX		2
#define ERR_STACK		4
#define ERR_COMPONLY	8
#define ERR_SYSTEM		0x10
#define ERR_USERCODE	0x20
#define ERR_COMMACC		0x40
#define ERR_CONTEXT 	0x80
#define ERR_USERCONTEXT	0xA0	//combination of both, save code memory
#define ERR_USERSTRING	0x21	//combination of both, save code memory
#define ERR_CANCEL		0x100
#define ERR_RUNTOKEN	0x200
#define ERR_NUMVAL		0x400
#define ERR_DIVZERO		0x800

#define KERNELSIZE		143
#define MAXUSER			168//*6 words p.struct=1008 words=4032 bytes=ca.2pages
#define MAXSTACK		64
#define MAXCODE			0xC00	// =3072 words =12288 bytes
								//takes 6 pages each 2048 bytes
#define CODETOP			MAXCODE - 1
#define USERTOP			(uint32_t)&gfUserHeader[MAXUSER]

#define RXBUFLEN		4096

#define DEFAULT_BAUD	11	//expanded to 115200, 
							//enter first 2 places, e.g. 96 for 9600
#define DEFAULT_NUMBASE	10 //decimal, enter 16 for HEX

#define CTRL_B  2
#define CTRL_D  4
#define BKSP  8
#define TAB   9
#define LFF  10		//for Macintosh OS: simply exchange values of CRR and LFF
#define CRR  13		//then New_Line will be interpreted correctly
#define CTRL_R  18
#define ESC  27
#define SPACE  32
#define BACKSLASH 92

#endif  //GLOBALDEFINES

// ##########################################################################

// Version changes:
//REVNUM 1: 29 October 2020 copied from CForth-G431-project REVNUM 48.
//			no USB implemented here to save current
//	08Nov20	return value of "interpretUserToken()" inverted
//			return of "interpretKernelToken()" changed to "void"
//			return of "executeTCode()" changed to "void"
//			opNODBG introduced,background process not executed,higher speed
//			if USART RX shorted during startup, AUTOEXE->"none"
//			(MINI: pull PA.. low during startup, AUTOEXE->"none")
//			subQuery() rejects (TIB bytes == 0), this simplifies scanNext()
//			return of scanNext() changed to "void", improvement of op_AUTO
//	26Nov20	Kernel Ops DEBUG, CASE, <CASE new, ENDIF replaced by THEN
//release 03Dec20
//REVNUM 2:
//	07Dec20	executeTcode etc. optimized for runtime improvement
//			Z-BREAK renamed ZBREAK and doZ-BRK renamed doZBRK
//			IF()...LOOP() and doIF()...doLOOP() improved
//			for better runtime behaviour and better readability of MEM()
//			;(SEMIS) optimized
//	10Dec20	pullup/down switched off when I/O is output (save current)
//	11Dec20	op_LED modes reorganized (modes 0,1,2,3)
//			user terminal communication during NO-DBG improved (CTRL_D)
//	16Dec20	time operators reorg. (MS, DS are blocking, BLOCK removed)		
//          improvement of CTRL_B (unblock MS and DS) behaviour
//  18Dec20 improvement of op_MS(),op_DECIS(): CTRL_D works in MS and DS
//			now 6 user projects in FLash
//release 28Dec20
//REVNUM 3:
// 20Jan21:	bugfix "RANA": assignment of ADC channels and number 
//release 22Jan21
//REVNUM 4:
//	23Jan21	automatic ADC oversampling introduced
//			LED mode can be modified from background process
//			SPEED introduced (3.68, 0.921MHz CPU clock selectable at runtime)
//	19MAY21	Kernel ops ROLL,INJECT,U> added, 2SWAP,2DUP removed, REV modified
//	22MAY21	%X and %D "Immediate,Background" now
//			op VAL reduced to Background Process value message
//	25May21	op_QUEST and AUTOEXE check moved to "popFlashProject" 
//			for better start of AUTOEXE when another project is loaded
//	29May21	Option LQFP48 introduced
//			I2C pins changed to PB7,8
//	30May21	PICK, ROLL, INJECT rearranged to Forth79 style
//			FS ("flush data stack") modified to "NO attributes"
//release 01 Jun 2021
//REVNUM 5:	14 Nov 2021
//			op_VAL improved (not restricted to STEP mode anymore)
//			PWM3 (PA15) new
//			bugfix load empty project with modified globals
//17Dec21	Option LQFP48 removed
//			MOT new
//			option CLOCK_14M new (3 speed grades: 921.6k, 3.6864M, 14.7456M)
//			gfUserToken[0] not deletable
//			AUTOEXE behaviour changed (-1 removed, index not increased)
//19Dec21	bugfix freq printout in op_QUEST()
//			code of subPWM1,2,3 straightened
//REVNUM 6:	08 May 2022
//08May22   %x renamed $, %d renamed %
//			preloaded V2 removed, now V1,P1,P2,P3 preloaded
//16May22	op_PRINTSTACK modified (CRR removed)
//27May22	string output of op_USER modified
//28May22	gfTICKCOUNT -> gfMSCOUNT and gfTIXCOUNT (decouple TIX from  MS,DS)
//07June22	VAR and CONST replaced by VARCON,
//			op_REV compiles op_doREV instead of op_doADDR, saves runtime
//			op_VAL removed, op_TRACE new
//			P1,P2,P3 renamed to K1,K2,K3,K4
//			due to these changes, some reorganisatuib iion of code
//09June22	struct "gfUserToken" renamed to "gfUserHeader"
//11June22	bugfix interpretNumber(), parsing big negative numbers
//			kernel operator ARRAY added
//13June22	improvement of compileUserToken() and restore()
//							(gfTCode offset calculation made more transparent)
//15June22	background process behaviour of KEY? and KEY improved
//release 17June2022
//REVNUM 7	18June2022
//18June22	bugfix gfKernelAttributes[], REV attribtute -->IB
//			subQuery(): max gfTibPos changed up to 80 + additional condition
//22March23	kernel op "NEG" renamend to "+/-"
//			kernel op "=" renamend to "=="
//30Sept23	kernel op "%" renamend to "DZ"
//			kernel op "$" renamend to "HX"
//			kernel ops UNTIL and correseponding doUNTIL added
//			kernel ops NOP, TIME, PULS and TRACE deleted
//			option "Cold Reset" (CTRL_R) removed
//			independent Watchdog removed
//03Oct23	P renamed . , PH renamed .H , P" renamed ."
//04Oct23	"RANA"(raed analog in) improved, new: disableADC(),saves current
//			Kernel Op SLEEP implemented
///			BSET, BCLR, BTST, .D implemented new
//			kernel Op 'Q' renamed to '?'
//07Oct23	numberBase saved in gerneral preset, always reactived at startup
//15Oct23	kernel op '&' renamed to 'AND', kernel op '|' renamed to 'OR
//17Oct23	USER output reformatted
//18Oct23	REPLACE and RESTORE improved
//21OCT23	REGW, REGBSET, REGBCLR, REGBTST stack parameters exchanged
//25OCT23	RANA renamed RADC, RDIG renamed RDIN, WAIT renamed TIME
//26Oct23	kernel ops "<<" and ">>" replaced by single "SHIFT"
//28Oct23	CONTI, BREAK, ZBREAK optimized, doCONTI,doBREAK,doZBREAK removed
//			K1,K2,K3,K4 renamed V1,V2,V3,V4
//30Oct23	servSystem() transferred from checkTerminal() to showPreStep()
//			gSYSCOUNT refresh rate 20Hz now, gLEDCOUNT adjusted to 300ms
//09Nov23	order of Kernel Ops rearranged
//12Nov23	RP-1 and RP-4 only "COMPILING" flag now.To be used ahead of RETURN
//release 12Nov23
//REVNUM 8:13Nov2023
//19DEC23	MEM attributes changed to A_BAK
//20Dec23	bugfix compileTCode: protection against CODETOP overflow
//21Dec23	digital port I/O improved
//23Dec23	14MHz crystal only 921kHz or 14M system frequency(3.68M cancelled)
//		   	op_QUEST(): printout of CPUclock improved
//released 25Dec23	
//REVNUM->9
//31Dec23	modification compilation user header with kernel name (op_colon)
//			new gfBackProcess=2: return to higher thread level with CTRL_R
//01Jan24	bugfix gfUserHeader[4] renamed to NSTP (confusion with op_MS)
//			op_MEM now sends block of 128 bytes(32 words)
//			changed: internally gfAuto = target UserFunction + 1
//			improvement countdown NSTP if stepper is not rotating
//04Jan24	PWM3 now at PB3
//			checkValidIO() and op_PINOH()...op_PINIZ() bugfix and reorg. pinNo
//07Jan23	number of Flash pages increased, more space for future code
//released 10Jan2024
//REVNUM-->10
//11Jan24	send0X32() modified: always with leading 0x even at small numbers
//18Jan24	bugfix op_SEE()
//19Jan24	decimal numbers can be entered with leading '%' at any muber base
//			see interpretNumber()
//07March24	kernel op RADC: only 1 single sample used for readout
//08March24	kernelOps ITX1 and IRXN implemented. IRXAN renamed to IADRXN
//09March24	reorg kernel op SAVE and LOAD more Forth typically
//12March24	I2C code made more failure-resistant
//released 14March2024
//REVNUM-->11
//31March24	bugfix cforth.h: DZ and HX have IMMEDIATE flag now
//02Apr24	doVALUE renamed doVAL

// ******************************

